import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import org.json.simple.JSONObject;
import org.json.simple.parser.ParseException;
import org.json.simple.parser.JSONParser;


public class HiairWebApi {
    public static void main(String []args){
        // ===== 設定 API Endpoint =====
        // 智慧簡訊
        String url_sendSMS = "https://api.sms.hinet.net/api/sendSMS/v1";
        // 短簡訊
        String url_sendShortSMS = "https://api.sms.hinet.net/api/sendShortSMS/v1";
        // 長簡訊
        String url_sendLongSMS = "https://api.sms.hinet.net/api/sendLongSMS/v1";
        // 查詢簡訊
        String url_querySMS = "https://api.sms.hinet.net/api/querySMS/v1";
        // 取消預約簡訊
        String url_cancelSMS = "https://api.sms.hinet.net/api/cancelSMS/v1";


        // ===== 設定 請求欄位 =====
        JSONObject data_base = new JSONObject();
        data_base.put("username", "請填入帳號");
        data_base.put("password", "請填入密碼");

        // --- 發送簡訊 ---
        // ex:
        // {
        // "username": "帳號",
        // "password": "密碼",
        // "mobile": "門號",
        // "message": "簡訊內容",
        // "limit_time": 1440,
        // "order_time": "220201103000"
        // }
        JSONObject data_send_base = new JSONObject(data_base);
        data_send_base.put("mobile", "請填入收訊手機號碼");
        data_send_base.put("message", "請填入簡訊內容");
        // 簡訊發送 - 一般
        JSONObject data_send_type_01 = new JSONObject(data_send_base);
        // 簡訊發送 - 設定 limit_time
        JSONObject data_send_type_02 = new JSONObject(data_send_base);
        data_send_type_02.put("limit_time", 1440);  // 1~1440，單位：分鐘。 (有效範圍外之數值，將預設為 1440 分鐘)
        // 簡訊發送 - 設定 order_time
        JSONObject data_send_type_03 = new JSONObject(data_send_base);
        data_send_type_03.put("order_time", "220627091500");  // 格式為 yyMMddHHmmss，yy：不含世紀之西元年，例如 2022 年請輸入 22。
        // 簡訊發送 - 設定 limit_time, order_time
        JSONObject data_send_type_04 = new JSONObject(data_send_base);
        data_send_type_04.put("limit_time", 1440);
        data_send_type_04.put("order_time", "220627091500");

        // --- 查詢簡訊、取消預約簡訊 ---
        // ex:
        // {
        // "username": "帳號",
        // "password": "密碼",
        // "message_id": " 簡訊 ID",
        // "msg_type": 2
        // }
        // 不指定簡訊類型
        JSONObject data_query_cancel_type_01 = new JSONObject(data_base);
        data_query_cancel_type_01.put("message_id", "請填入簡訊ID");  // ex: A8784619060309156XXX
        // 指定簡訊類型
        JSONObject data_query_cancel_type_02 = new JSONObject(data_base);
        data_query_cancel_type_02.put("message_id", "請填入簡訊ID");  // ex: A8784619060309156XXX
        data_query_cancel_type_02.put("msg_type", 12);  // 2:查詢/取消短簡訊 or 12:查詢/取消長簡訊

        // check data
//        System.out.println(data_send_type_01.toJSONString());
//        System.out.println(data_send_type_02.toJSONString());
//        System.out.println(data_send_type_03.toJSONString());
//        System.out.println(data_send_type_04.toJSONString());
//        System.out.println(data_query_cancel_type_01.toJSONString());
//        System.out.println(data_query_cancel_type_02.toJSONString());


        // ===== 指定 請求欄位 =====
        // --- 發送簡訊 ---
        String url = url_sendSMS;
//        String url = url_sendShortSMS;
//        String url = url_sendLongSMS;
        JSONObject data = data_send_type_01;
//        JSONObject data = data_send_type_02;
//        JSONObject data = data_send_type_03;
//        JSONObject data = data_send_type_04;

        // --- 查詢簡訊、取消預約簡訊 ---
//        String url = url_querySMS;
//        String url = url_cancelSMS;
//        JSONObject data = data_query_cancel_type_01;
//        JSONObject data = data_query_cancel_type_02;


        // ===== 發送請求 =====
        Pair<Object, Object> result = executePost(url, data.toString());
        if (result != null) {
            System.out.println(result.http_code);
            System.out.println(result.ret_data);
            JSONParser parser = new JSONParser();
            try{
                Object obj = parser.parse(result.ret_data.toString());
                JSONObject ret_object = (JSONObject)obj;
                if ((int) result.http_code == 200) {
                    if ((long) ret_object.get("ret_code") == 0) {
                        System.out.println(ret_object.get("ret_code"));
                        System.out.println(ret_object.get("ret_content"));
                        System.out.println(ret_object.get("msg_type"));
                    } else {
                        System.out.println(ret_object.get("ret_code"));
                        System.out.println(ret_object.get("ret_content"));
                    }
                } else {
                    System.out.println(result.http_code);
                    System.out.println(ret_object.get("ret_code"));
                    System.out.println(ret_object.get("ret_content"));
                }
            }catch(ParseException pe){
                System.out.println("position: " + pe.getPosition());
                System.out.println(pe);
            }
        } else {
            // inner error
            System.out.println("[inner error] check what error printed.");
        }
    }

    public static Pair<Object, Object> executePost(String targetURL, String urlParameters) {
        HttpURLConnection connection = null;

        try {
            //Create connection
            URL url = new URL(targetURL);
            connection = (HttpURLConnection) url.openConnection();
            connection.setRequestMethod("POST");
            connection.setRequestProperty("Content-Type", "application/json");
            connection.setUseCaches(false);
            connection.setDoOutput(true);
            connection.setConnectTimeout(30000);  // 30秒
            connection.setReadTimeout(30000);

            //Send request
            DataOutputStream wr = new DataOutputStream (
                    connection.getOutputStream());
            wr.write(urlParameters.getBytes(StandardCharsets.UTF_8));
            wr.close();

            //Get Response
            InputStream is;
            try {
                is = connection.getInputStream();
            } catch(IOException ex) {
                // 非 200 會進到catch
                is = connection.getErrorStream();
                if (is == null) {
                    throw ex;
                }
            }
            BufferedReader rd = new BufferedReader(new InputStreamReader(is, StandardCharsets.UTF_8));
            StringBuilder response = new StringBuilder();
            String line;
            while ((line = rd.readLine()) != null) {
                response.append(line);
                response.append(System.lineSeparator());
            }
            rd.close();

            //Get Response code
            int code = connection.getResponseCode();
            return new Pair(code, response.toString());
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            if (connection != null) {
                connection.disconnect();
            }
        }
    }

    public static class Pair<U, V> {

        /**
         * The first element of this <code>Pair</code>
         */
        private U http_code;

        /**
         * The second element of this <code>Pair</code>
         */
        private V ret_data;

        /**
         * Constructs a new <code>Pair</code> with the given values.
         *
         * @param http_code  the first element
         * @param ret_data the second element
         */
        public Pair(U http_code, V ret_data) {

            this.http_code = http_code;
            this.ret_data = ret_data;
        }
    }

}

